@@ -5,6 +5,7 @@
 """Weblate API client library."""
 
 from __future__ import annotations
+
 import json
 import logging
 from copy import copy
@@ -14,11 +15,11 @@ from urllib.parse import urlencode, urlparse
 import dateutil.parser
 import requests
 from requests.adapters import HTTPAdapter
-from requests.packages.urllib3.util.retry import Retry
+from requests.packages.urllib3.util.retry import Retry  # type: ignore[import-not-found]
 
 log = logging.getLogger("wlc")
 
-__version__ = "1.15"
+__version__ = "1.16.1"
 
 URL = "https://weblate.org/"
 DEVEL_URL = "https://github.com/WeblateOrg/wlc"
@@ -31,18 +32,18 @@ TIMESTAMPS = {"last_change"}
 class WeblateException(Exception):
     """Generic error."""
 
-    def __init__(self, message: str | None = None):
+    def __init__(self, message: str | None = None) -> None:
         super().__init__(message or self.__doc__)
 
 
 class WeblateThrottlingError(WeblateException):
     """Throttling on the server."""
 
-    def __init__(self, limit: str, retry_after: str):
+    def __init__(self, limit: str, retry_after: str) -> None:
         self.limit = limit
         self.retry_after = retry_after
         message_segments = [
-            cast(str, self.__doc__)
+            cast("str", self.__doc__)
         ]  # workaround for https://github.com/python/mypy/issues/15825
         if limit:
             message_segments.append(f"Limit is {limit} requests.")
@@ -72,7 +73,7 @@ class Weblate:
         method_whitelist: Collection[str] | None = None,
         backoff_factor: int = 0,
         timeout: int = 300,
-    ):
+    ) -> None:
         """Create the object, storing key, API url and requests retry args."""
         self.session = requests.Session()
         if config is not None:
@@ -117,11 +118,21 @@ class Weblate:
     def permission_error_message(error):
         """Get detail from serialized DRF PermissionDenied exception."""
         try:
-            return error.response.json()["detail"]
-        except (json.JSONDecodeError, KeyError):
+            response_json = error.response.json()
+        except json.JSONDecodeError:
             return None
 
-    def process_error(self, error):
+        # Since Weblate 5.10
+        if "errors" in response_json:
+            return ", ".join(error["detail"] for error in response_json["errors"])
+
+        # Weblate before 5.10
+        if "detail" in response_json:
+            return response_json["detail"]
+
+        return None
+
+    def process_error(self, error) -> None:
         """Raise WeblateException for known HTTP errors."""
         if isinstance(error, requests.HTTPError):
             status_code = error.response.status_code
@@ -129,7 +140,8 @@ class Weblate:
             if status_code == 429:
                 headers = error.response.headers
                 raise WeblateThrottlingError(
-                    headers.get("X-RateLimit-Limit"), headers.get("Retry-After")
+                    headers.get("X-RateLimit-Limit", "unknown"),
+                    headers.get("Retry-After", "unknown"),
                 ) from error
             if status_code == 404:
                 raise WeblateException(
@@ -153,7 +165,7 @@ class Weblate:
             reason = error.response.reason
             try:
                 error_string = str(error.response.json())
-            except Exception:
+            except Exception:  # noqa: BLE001
                 error_string = ""
             raise WeblateException(
                 f"HTTP error {status_code}: {reason} {error_string}"
@@ -221,7 +233,7 @@ class Weblate:
 
     def _post_factory(self, prefix, path, kwargs):
         """Wrapper for posting objects."""
-        return self.post("/".join((prefix, path, "")), **kwargs)
+        return self.post(f"{prefix}/{path}/", **kwargs)
 
     def get(self, path, params=None):
         """Perform GET request on the API."""
@@ -238,7 +250,7 @@ class Weblate:
 
     def _get_factory(self, prefix, path, parser):
         """Wrapper for getting objects."""
-        data = self.get("/".join((prefix, path, "")))
+        data = self.get(f"{prefix}/{path}/")
         return parser(weblate=self, **data)
 
     def get_object(self, path):
@@ -357,7 +369,7 @@ class Weblate:
 
     @staticmethod
     def _should_verify_ssl(path):
-        """Cheks if it should verify ssl certificates."""
+        """Checks if it should verify ssl certificates."""
         url = urlparse(path)
         is_localhost = url.netloc.startswith(LOCALHOST_NETLOC)
         return url.scheme == "https" and (not is_localhost)
@@ -368,30 +380,31 @@ class LazyObject(dict):
 
     PARAMS: ClassVar[tuple[str, ...]] = ()
     OPTIONALS: ClassVar[set[str]] = set()
+    NULLS: ClassVar[set[str]] = set()
     MAPPINGS: ClassVar[dict[str, Any]] = {}
     ID: ClassVar[str] = "url"
 
-    def __init__(self, weblate, url, **kwargs):
+    def __init__(self, weblate, url, **kwargs) -> None:
         """Construct object for given Weblate instance."""
         super().__init__()
         self.weblate = weblate
         self._url = url
-        self._data = {}
+        self._data: dict[str, Any] = {}
         self._loaded = False
-        self._attribs = {}
+        self._attribs: dict[str, Any] = {}
         self._load_params(**kwargs)
         self._load_params(url=url)
 
     def get_data(self):
         return copy(self._data)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return str(self._data)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return repr(self._data)
 
-    def _load_params(self, **kwargs):
+    def _load_params(self, **kwargs) -> None:
         for param in self.PARAMS:
             if param in kwargs:
                 value = kwargs[param]
@@ -410,14 +423,14 @@ class LazyObject(dict):
         for key, value in kwargs.items():
             self._attribs[key] = value
 
-    def ensure_loaded(self, attrib):
+    def ensure_loaded(self, attrib) -> None:
         """Ensure attribute is loaded from remote."""
         if attrib in self._data or attrib in self._attribs:
             return
         if not self._loaded:
             self.refresh()
 
-    def refresh(self):
+    def refresh(self) -> None:
         """Read object again from remote."""
         data = self.weblate.get(self._url)
         self._load_params(**data)
@@ -431,9 +444,11 @@ class LazyObject(dict):
         try:
             return self._data[name]
         except KeyError as error:
+            if name in self.NULLS:
+                return None
             raise AttributeError(name) from error
 
-    def setattrvalue(self, name, value):
+    def setattrvalue(self, name, value) -> None:
         if name not in self.PARAMS:
             raise AttributeError(name)
 
@@ -442,7 +457,7 @@ class LazyObject(dict):
     def __getitem__(self, name):
         return self.__getattr__(name)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(list(self.keys()))
 
     def keys(self):
@@ -451,7 +466,11 @@ class LazyObject(dict):
         if len(self._data) <= 1:
             self.refresh()
         for param in self.PARAMS:
-            if param not in self.OPTIONALS or param in self._data:
+            if (
+                param not in self.OPTIONALS
+                or param in self._data
+                or param in self.NULLS
+            ):
                 yield param
 
     def items(self):
@@ -598,12 +617,15 @@ class Project(LazyObject, RepoObjectMixin):
         self.ensure_loaded("categories_url")
         return self.weblate.list_categories(self._attribs["categories_url"])
 
-    def delete(self):
+    def delete(self) -> None:
         self.weblate.raw_request("delete", self._url)
 
     def create_component(self, **kwargs):
         return self.weblate.create_component(self.slug, **kwargs)
 
+    def full_slug(self):
+        return self.slug
+
 
 class Category(LazyObject):
     """Category object."""
@@ -611,6 +633,14 @@ class Category(LazyObject):
     PARAMS: ClassVar[tuple[str, ...]] = ("category", "name", "project", "slug", "url")
     MAPPINGS: ClassVar[dict[str, Any]] = {"project": Project}
 
+    def full_slug(self):
+        current = self
+        slugs = [self.project.slug, self.slug]
+        while current.category:
+            current = current.category
+            slugs.insert(1, current.slug)
+        return "/".join(slugs)
+
 
 class Component(LazyObject, RepoObjectMixin):
     """Component object."""
@@ -645,6 +675,7 @@ class Component(LazyObject, RepoObjectMixin):
         "category",
         "linked_component",
     }
+    NULLS: ClassVar[set[str]] = {"category"}
     ID: ClassVar[str] = "slug"
     MAPPINGS: ClassVar[dict[str, Any]] = {
         "category": Category,
@@ -653,6 +684,11 @@ class Component(LazyObject, RepoObjectMixin):
     }
     REPOSITORY_CLASS = Repository
 
+    def full_slug(self) -> str:
+        if self.category:
+            return f"{self.category.full_slug()}/{self.slug}"
+        return f"{self.project.full_slug()}/{self.slug}"
+
     def list(self):
         """List translations in the component."""
         self.ensure_loaded("translations_url")
@@ -693,7 +729,7 @@ class Component(LazyObject, RepoObjectMixin):
         self.ensure_loaded("changes_list_url")
         return self.weblate.list_changes(self._attribs["changes_list_url"])
 
-    def delete(self):
+    def delete(self) -> None:
         self.weblate.raw_request("delete", self._url)
 
     def add_source_string(self, msgid, msgstr):
@@ -786,7 +822,7 @@ class Translation(LazyObject, RepoObjectMixin):
 
         return self.weblate.request("post", url, files=files, data=kwargs)
 
-    def delete(self):
+    def delete(self) -> None:
         self.weblate.raw_request("delete", self._url)
 
     def units(self, **kwargs):
