@@ -8,50 +8,55 @@ from __future__ import annotations
 
 import os.path
 from configparser import NoOptionError, RawConfigParser
+from typing import TYPE_CHECKING
 
-from xdg.BaseDirectory import load_config_paths
+from xdg.BaseDirectory import load_config_paths  # type: ignore[import-untyped]
 
 import wlc
 
+if TYPE_CHECKING:
+    from collections.abc import Generator
+
 __all__ = ["NoOptionError", "WeblateConfig"]
 
 
 class WeblateConfig(RawConfigParser):
     """Configuration parser wrapper with defaults."""
 
-    def __init__(self, section="weblate"):
+    def __init__(self, section="weblate") -> None:
         """Construct WeblateConfig object."""
         super().__init__(delimiters=("=",))
         self.section = section
         self.set_defaults()
 
-    def set_defaults(self):
+    def set_defaults(self) -> None:
         """Set default values."""
         self.add_section("keys")
         self.add_section(self.section)
         self.set(self.section, "key", "")
         self.set(self.section, "url", wlc.API_URL)
-        self.set(self.section, "retries", 0)
-        self.set(self.section, "timeout", 300)
+        self.set(self.section, "retries", "0")
+        self.set(self.section, "timeout", "300")
         self.set(self.section, "status_forcelist", None)
         self.set(
             self.section, "method_whitelist", "HEAD\nTRACE\nDELETE\nOPTIONS\nPUT\nGET"
         )
-        self.set(self.section, "backoff_factor", 0)
+        self.set(self.section, "backoff_factor", "0")
 
     @staticmethod
-    def find_configs():
+    def find_configs() -> Generator[str]:
         # Handle Windows specifically
-        if "APPDATA" in os.environ:
-            win_path = os.path.join(os.environ["APPDATA"], "weblate.ini")
-            if os.path.exists(win_path):
-                yield win_path
+        for envname in ("APPDATA", "LOCALAPPDATA"):
+            if path := os.environ.get(envname):
+                win_path = os.path.join(path, "weblate.ini")
+                if os.path.exists(win_path):
+                    yield win_path
 
         # Generic XDG paths
         yield from load_config_paths("weblate")
         yield from load_config_paths("weblate.ini")
 
-    def load(self, path=None):
+    def load(self, path=None) -> None:
         """Load configuration from XDG paths."""
         if path is None:
             path = list(self.find_configs())
